#include <string>
#include <vector>
using std::string;
using std::vector;

#include <Ogre.h>

#include "World.h"
#include "Element.h"

World::World(Ogre::SceneManager *smgr, string name)
{
  Ogre::ColourValue fadeColour( 0.93, 0.8, 0.5 );
  Ogre::ColourValue ambientColour( 0.5, 0.5, 0.5 );

  /* up the lighters ! */
  smgr->setAmbientLight(ambientColour);

  /* Set fog */
  smgr->setFog(Ogre::FOG_LINEAR, fadeColour, 0.001, 500, 1000);
  // XXX vp->setBackgroundColour( fadeColour );

  /* Now load terrain */
  smgr->setWorldGeometry(name + ".terrain");

  /* Ray for World collision */
  vRay = smgr->createRayQuery( Ogre::Ray() );
  

}


World::~World()
{
  delete vRay;
}


Ogre::RaySceneQuery* World::getRay()
{
  return vRay;
}

void World::applyGravity(vector<Element*> elements)
{
  vector<Element*>::iterator iter;
  
  for (iter = elements.begin(); iter != elements.end(); iter++) {

    static Ogre::Ray updateRay;
    Element *e = (Element*)(*iter);
    Ogre::SceneNode *s = e->getSceneNode();

    updateRay.setOrigin(s->getPosition());
    updateRay.setDirection(Ogre::Vector3::NEGATIVE_UNIT_Y);
    vRay->setRay(updateRay);
    Ogre::RaySceneQueryResult& qryResult = vRay->execute();
    Ogre::RaySceneQueryResult::iterator i = qryResult.begin();

    if (i != qryResult.end() && i->worldFragment)
      {
	Ogre::SceneQuery::WorldFragment* wf = i->worldFragment;
	s->setPosition(s->getPosition().x, 
		       i->worldFragment->singleIntersection.y + 10, 
		       s->getPosition().z);
      }

  }


}
